#!/bin/bash 
#===============================================================================
#
#          FILE: collectLogs.sudo-nopass.sshpass.sh
#
#         USAGE: ./collectLogs.sudo-nopass.sshpass.sh
#
#   DESCRIPTION: Collects Logs directory from linux based NAC/NES/NAG systems by connecting to the remote machine using SSH to tar the files and then copy them to your workstation using scp. 
#
#   NOTES:
#       This script assumes that you cannot connect directly as nolio_user. 
#       Instead, it assumes that you need to connect as loginuser and then have the abililty to "sudo su - <nolio_user>" without being prompted for a password.
#           This script will not work if you get prompted by sudo for a password.
#
#       This script assumes that you DO get prompted for a password when running SSH and SCP commands from the machine where this 
#           script is running -> to the EOF list of hosts. 
#           This script prompts once for the password (not clear text), puts it into a variable, and then passes to sshpass when connecting (via ssh) and copying (via scp).
#
#       This script assumes that you are able to connect directly to the <EOF list of hosts> with the user that owns the nolio_home folder (defined below).
#
#       If any of the following items are different across the machines you want to collect data from then you will need to copy the script & adjust the necessary variables:
#           - the ssh password used by nolio_user on all EOF list of hosts.
#           - nolio_user
#           - nolio_home
#               For example: 
#                   It is common for the NAC and NES to share folder install paths. So you could use this one script to collect logs from your nac and all nes. However, 
#                   agents are not installed (by default) to the same folder location as nac and nes. So if you're collecting files from agents then you will 
#                   need to copy the script and adjust the nolio_home variable. 
#           
#       If you want to setup your workstation so that it can connect to the remote machine without being prompted for a password then run the setupSSH.sh script - then 
#           use the collectLogs.no.sshpass.sh. 
#           NOTE: While running setupSSH.sh, if you specify a passphrase then this script will not work. This script does not work with ssh passphrases. Instead, 
#                   use the collectLogs.no.sshpass.sh. It will prompt you for the passphrase.
#
#   REQUIREMENTS:
#       1. the script be run from a mac or linux machine.
#       2. the ssh and scp commands are available.
#       3. the loginuser specified below must be configured so that it can "sudo su - <nolio_user>" without being prompted for a password.
#         
#===============================================================================

set -eu -o pipefail

function ensure_app_runnable() {
    app=${1+$1}
    which $app >/dev/null || {
        echo "App $app not found on the PATH"
        exit 1
    }
}

ensure_app_runnable ssh
ensure_app_runnable scp
ensure_app_runnable sshpass

loginuser=myuser
nolio_user=noliouser
nolio_home="/opt/CA/ReleaseAutomationServer"

timestamp=$(date +%Y-%m-%d-%H-%M-%S)

# Used for password prompted method
read -s -p "Enter password for user '$loginuser' that will be used for SSH/SCP access: " noliopass

while read host
do
    printf '\n%s\n' "Gathering data for: ${host}"

    # Archive filename
    arname="nolio_${host}_${timestamp}.tar.gz"

    # Used with sshkeys that don't prompt for password/passphrase
    # NOTE: the -n redirects stdin from /dev/null. If this is not done then SSH would also read stdin and eat the stdin used by reading the heredoc. Some other options are discussed here: 
    # https://unix.stackexchange.com/questions/107800/using-while-loop-to-ssh-to-multiple-servers

    # Connect to remote host and tar the nimi_config.xml and logs folder.
    sshpass -p "$noliopass" ssh -n ${loginuser}@${host} "sudo su - ${nolio_user} -c \"cd ${nolio_home} && tar zcf ${arname} conf/nimi_config.xml ./logs && chmod 777 ${nolio_home}/${arname}\""

    # Copy the tar file from the remote machine to the local folder where this script is running.
    sshpass -p "$noliopass" scp ${loginuser}@${host}:${nolio_home}/${arname} .

done <<EOF
myhost1.fqdn
myhost2.resolvable.name.or.ip
EOF

